package xyz.lychee.lagfixer.libs.adventure.bossbar;

import java.util.Set;
import xyz.lychee.lagfixer.libs.adventure.audience.Audience;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.ComponentLike;
import xyz.lychee.lagfixer.libs.adventure.util.Index;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.UnmodifiableView;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.OverrideOnly;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.ScheduledForRemoval;
import xyz.lychee.lagfixer.libs.examination.Examinable;

@NonExtendable
public interface BossBar extends Examinable {
   float MIN_PROGRESS = 0.0F;
   float MAX_PROGRESS = 1.0F;
   /** @deprecated */
   @Deprecated
   @ScheduledForRemoval(
      inVersion = "5.0.0"
   )
   float MIN_PERCENT = 0.0F;
   /** @deprecated */
   @Deprecated
   @ScheduledForRemoval(
      inVersion = "5.0.0"
   )
   float MAX_PERCENT = 1.0F;

   static @NotNull BossBar bossBar(final @NotNull ComponentLike name, final float progress, @NotNull final @NotNull Color color, @NotNull final @NotNull Overlay overlay) {
      BossBarImpl.checkProgress(progress);
      return bossBar(name.asComponent(), progress, color, overlay);
   }

   static @NotNull BossBar bossBar(final @NotNull Component name, final float progress, @NotNull final @NotNull Color color, @NotNull final @NotNull Overlay overlay) {
      BossBarImpl.checkProgress(progress);
      return new BossBarImpl(name, progress, color, overlay);
   }

   static @NotNull BossBar bossBar(final @NotNull ComponentLike name, final float progress, @NotNull final @NotNull Color color, @NotNull final @NotNull Overlay overlay, final @NotNull Set<Flag> flags) {
      BossBarImpl.checkProgress(progress);
      return bossBar(name.asComponent(), progress, color, overlay, flags);
   }

   static @NotNull BossBar bossBar(final @NotNull Component name, final float progress, @NotNull final @NotNull Color color, @NotNull final @NotNull Overlay overlay, final @NotNull Set<Flag> flags) {
      BossBarImpl.checkProgress(progress);
      return new BossBarImpl(name, progress, color, overlay, flags);
   }

   @NotNull Component name();

   @Contract("_ -> this")
   default @NotNull BossBar name(final @NotNull ComponentLike name) {
      return this.name(name.asComponent());
   }

   @Contract("_ -> this")
   @NotNull BossBar name(final @NotNull Component name);

   float progress();

   @Contract("_ -> this")
   @NotNull BossBar progress(final float progress);

   /** @deprecated */
   @Deprecated
   @ScheduledForRemoval(
      inVersion = "5.0.0"
   )
   default float percent() {
      return this.progress();
   }

   /** @deprecated */
   @Deprecated
   @ScheduledForRemoval(
      inVersion = "5.0.0"
   )
   @Contract("_ -> this")
   default @NotNull BossBar percent(final float progress) {
      return this.progress(progress);
   }

   @NotNull
   @NotNull Color color();

   @Contract("_ -> this")
   @NotNull BossBar color(@NotNull final @NotNull Color color);

   @NotNull
   @NotNull Overlay overlay();

   @Contract("_ -> this")
   @NotNull BossBar overlay(@NotNull final @NotNull Overlay overlay);

   @UnmodifiableView @NotNull Set<Flag> flags();

   @Contract("_ -> this")
   @NotNull BossBar flags(final @NotNull Set<Flag> flags);

   boolean hasFlag(@NotNull final @NotNull Flag flag);

   @Contract("_ -> this")
   @NotNull BossBar addFlag(@NotNull final @NotNull Flag flag);

   @Contract("_ -> this")
   @NotNull BossBar removeFlag(@NotNull final @NotNull Flag flag);

   @Contract("_ -> this")
   @NotNull BossBar addFlags(final @NotNull Flag... flags);

   @Contract("_ -> this")
   @NotNull BossBar removeFlags(final @NotNull Flag... flags);

   @Contract("_ -> this")
   @NotNull BossBar addFlags(final @NotNull Iterable<Flag> flags);

   @Contract("_ -> this")
   @NotNull BossBar removeFlags(final @NotNull Iterable<Flag> flags);

   @Contract("_ -> this")
   @NotNull BossBar addListener(@NotNull final @NotNull Listener listener);

   @Contract("_ -> this")
   @NotNull BossBar removeListener(@NotNull final @NotNull Listener listener);

   @UnmodifiableView @NotNull Iterable<? extends BossBarViewer> viewers();

   default @NotNull BossBar addViewer(final @NotNull Audience viewer) {
      viewer.showBossBar(this);
      return this;
   }

   default @NotNull BossBar removeViewer(final @NotNull Audience viewer) {
      viewer.hideBossBar(this);
      return this;
   }

   public static enum Overlay {
      PROGRESS("progress"),
      NOTCHED_6("notched_6"),
      NOTCHED_10("notched_10"),
      NOTCHED_12("notched_12"),
      NOTCHED_20("notched_20");

      public static final Index<String, Overlay> NAMES = Index.create(Overlay.class, (overlay) -> {
         return overlay.name;
      });
      private final String name;

      private Overlay(final String name) {
         this.name = name;
      }
   }

   public static enum Flag {
      DARKEN_SCREEN("darken_screen"),
      PLAY_BOSS_MUSIC("play_boss_music"),
      CREATE_WORLD_FOG("create_world_fog");

      public static final Index<String, Flag> NAMES = Index.create(Flag.class, (flag) -> {
         return flag.name;
      });
      private final String name;

      private Flag(final String name) {
         this.name = name;
      }
   }

   public static enum Color {
      PINK("pink"),
      BLUE("blue"),
      RED("red"),
      GREEN("green"),
      YELLOW("yellow"),
      PURPLE("purple"),
      WHITE("white");

      public static final Index<String, Color> NAMES = Index.create(Color.class, (color) -> {
         return color.name;
      });
      private final String name;

      private Color(final String name) {
         this.name = name;
      }
   }

   @OverrideOnly
   public interface Listener {
      default void bossBarNameChanged(final @NotNull BossBar bar, final @NotNull Component oldName, final @NotNull Component newName) {
      }

      default void bossBarProgressChanged(final @NotNull BossBar bar, final float oldProgress, final float newProgress) {
         this.bossBarPercentChanged(bar, oldProgress, newProgress);
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      default void bossBarPercentChanged(final @NotNull BossBar bar, final float oldProgress, final float newProgress) {
      }

      default void bossBarColorChanged(final @NotNull BossBar bar, @NotNull final @NotNull Color oldColor, @NotNull final @NotNull Color newColor) {
      }

      default void bossBarOverlayChanged(final @NotNull BossBar bar, @NotNull final @NotNull Overlay oldOverlay, @NotNull final @NotNull Overlay newOverlay) {
      }

      default void bossBarFlagsChanged(final @NotNull BossBar bar, final @NotNull Set<Flag> flagsAdded, final @NotNull Set<Flag> flagsRemoved) {
      }
   }
}
